/*
		BitvectorSel.cpp: Implementation of the BitvectorSel class

        Copyright (C) 2000 Patrick Dughi

        This software may be used and distributed according to the terms
        of the GNU Public License, incorporated herein by reference, located
		as the file gpl.txt in this same directory.

        The author may be reached at dughi@imaxx.net.

        A nice bitvector control class.
*/
#include "stdafx.h"
#include "Cbreak.h"
#include "BitvectorSel.h"
#include "utils.h"
#include "ConstantLoader.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// BitvectorSel dialog

extern ConstantLoader cl;

BitvectorSel::BitvectorSel(int def_val, char *aname, CWnd* pParent /*=NULL*/)
	: CDialog(BitvectorSel::IDD, pParent)
{
  total=def_val;
  name = aname;
	//{{AFX_DATA_INIT(BitvectorSel)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

}


void BitvectorSel::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(BitvectorSel)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(BitvectorSel, CDialog)
	//{{AFX_MSG_MAP(BitvectorSel)
	ON_BN_CLICKED(IDC_BIT_CLEAR, OnBitClear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// BitvectorSel message handlers

BOOL BitvectorSel::OnInitDialog() 
{
	CDialog::OnInitDialog();
	CWnd *button,*last=this->GetDlgItem(IDC_BIT_F11);
	CRect tail,tip;
	int i,r;
	map<string,int>::iterator hex;

  this->SetWindowText(name.c_str());
	this->GetWindowRect(&tip);
	last->GetWindowRect(&tail);

	/* member, 0,0 is the top left of the screen */
	okay_height = tip.bottom - tail.top;

	/* now, we set up our relative button locations */
  button=this->GetDlgItem(IDC_BIT_CLEAR);
	button->GetWindowRect(&rel_button_loc[0]);
  button=this->GetDlgItem(IDOK);
	button->GetWindowRect(&rel_button_loc[1]);
  button=this->GetDlgItem(IDCANCEL);
	button->GetWindowRect(&rel_button_loc[2]);

	 /* make relative. We only use top left for positioning, and left stays the same */
  rel_button_loc[0].top = tip.bottom - rel_button_loc[0].top;
  rel_button_loc[1].top = tip.bottom - rel_button_loc[1].top;
  rel_button_loc[2].top = tip.bottom - rel_button_loc[2].top;

	/* now, activate only those which are used */
  if(cl.constant_strings.find(name.c_str()) == cl.constant_strings.end()) {
	  i=0;
  } else {
    for(i=IDC_BITV1,hex=cl.constant_strings[name].enmap.begin();i <= IDC_BITV32;i++) {
 		  if(GetDlgItem(i)) {

		  	if(hex == cl.constant_strings[name].enmap.end()) {
			 	  ((CButton *)GetDlgItem(i))->ShowWindow(SW_HIDE);
				} else {
				  ((CButton *)GetDlgItem(i))->SetWindowText(hex->first.c_str());

			  	if((hex->second) & total) {
            ((CButton *)GetDlgItem(i))->SetCheck(1);
					}
					hex++;
				}
			}
		}	
  }
	/* now, size it */
  i = IDC_BIT_F1 + (cl.constant_strings[name].enmap.size())/3;
	button=this->GetDlgItem(i);
	button->GetWindowRect(&tail);
  r = tail.top - tip.top;
	this->SetWindowPos(NULL,tip.left,tip.top,
											tip.left + tip.right,r+okay_height,SWP_NOMOVE|SWP_NOZORDER);
	button=this->GetDlgItem(IDC_BIT_CLEAR);
  button->SetWindowPos(NULL,rel_button_loc[0].left,r,
		0,0,SWP_NOSIZE |SWP_NOZORDER);
	button->ShowWindow(SW_SHOW);
	button=this->GetDlgItem(IDOK);
  button->SetWindowPos(NULL,rel_button_loc[1].left,r,
		0,0,SWP_NOSIZE |SWP_NOZORDER);
	button->ShowWindow(SW_SHOW);
	button=this->GetDlgItem(IDCANCEL);
  button->SetWindowPos(NULL,rel_button_loc[2].left,r,
		0,0,SWP_NOSIZE |SWP_NOZORDER);
  button->ShowWindow(SW_SHOW);
	return TRUE;
    	
}

void BitvectorSel::OnOK() 
{
  int i,r;
	total=0;
	char text_string[2056];
	string better;

	for(i=IDC_BITV1,r=0;i <= IDC_BITV32;i++) {
		if(GetDlgItem(i)) {
			if(((CButton *)GetDlgItem(i))->GetCheck() == 1) {
				 ((CButton *)GetDlgItem(i))->GetWindowText(text_string,2056);
				 better=text_string;
				 total +=  cl.constant_strings[name].enmap[better];
			} 
		}
	}
  TRACE0(iprintf("Total: %d\n",total));

	CDialog::OnOK();
}

void BitvectorSel::OnBitClear() 
{
	int idnum;

	for(idnum=IDC_BITV1;idnum <= IDC_BITV32;idnum++) {
		if(GetDlgItem(idnum)) {
			((CButton *)GetDlgItem(idnum))->SetCheck(BST_UNCHECKED);
		}
	}
  
}
