/*
		Mob.cpp : implementation file

        Copyright (C) 2000 Patrick Dughi

        This software may be used and distributed according to the terms
        of the GNU Public License, incorporated herein by reference, located
		as the file gpl.txt in this same directory.

        The author may be reached at dughi@imaxx.net.
.		
		Mob class structure and related functions (load/write/etc)
*/
#pragma warning(disable:4786)

#include "stdafx.h"
#include "Cbreak.h"
#include "Mob.h"
#include "utils.h"
#include "CircleFile.h"
#include <string.h>
#include <ctype.h>
#include <map>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Mob::Mob()
{
  int i;
	for(i=0;i<NUM_OF_STATS;i++) {
    stats[i] = 14;
	}
  stats[STAT_STRADD]=attack_type=0;
}

Mob::~Mob()
{

}

int Mob::ascii2bitvector(const char *input)
{
  int i;
	int bitvector=0;

	if(isdigit(input[0])) {
		return (atoi(input));
	}
	for(i=0;i<strlen(input);i++) {
		if(input[i] >= 97 && input[i] <= 122) {
			bitvector |= 1 << (input[i] - 'a');
		} else {
			bitvector |= 1 << ((input[i] - 'A')+26);
		}
	}
	return bitvector;
}

void Mob::bitvector2ascii(char *output, int bitvector)
{
  int i, j = 0;
  char *flags="abcdefghijklmnopqrstuvwxyzABCDEF";

  for (i = 0; flags[i] != '\0'; i++) {
	  if (bitvector & (1 << i)) {
        output[j++] = flags[i];
    }
	}

  if (j == 0) {
    output[j++] = '0';
  }

  output[j]='\0';
  return;
}

void crlf_clean(string *dirty);

int Mob::load_mob(CircleFile *cf)
{
  map<int,Zone_Data>::const_iterator zi;
	int found=0,bad=0;
  extern World the_world;
  string temp,temp2;
  int type=0,i;


  /* lets set their stats briefly */
  for(i=0;i<NUM_OF_STATS;i++) {
	  stats[i]=14;
  }
	stats[STAT_STRADD]=0;
  /* have to verify that each mob is _in_ a zone */
  for(zi=the_world.zone_list.begin();zi!=the_world.zone_list.end() && !found;zi++) {
	  if(zi->first * 100 <= number && zi->second.top >= number) {
		  found=1;
		}
  }

	if(!found) {
		TRACE0(iprintf("Mob %d not found in a zone.\n",number));
		bad=1;
	}
 
  zone = cf->the_zone;
  cf->tilde_read(&temp);
	alias=temp;
	cf->tilde_read(&temp);
	sdesc=temp;
	cf->tilde_read(&temp);
	ldesc=temp;
	cf->tilde_read(&temp);
	ddesc=temp;

	/* fun, action, affection, alignment, type */
  cf->get_line(&temp);

	temp2=temp.substr(0,temp.find(" "));
	action=ascii2bitvector(temp2.c_str());

  temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
	affect=ascii2bitvector(temp2.c_str());

  temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
	alignment=atoi(temp2.c_str());

	temp=temp.substr(temp.find(" ")+1);
	if(temp[0] == 'E') {
		 type=1;
	}

	/* great. level, thac0, ac, hitpoints & attack damage */

  cf->get_line(&temp);
	temp2=temp.substr(0,temp.find(" "));
	level = atoi(temp2.c_str());

	temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
  thac0=atoi(temp2.c_str());

	temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
  ac=atoi(temp2.c_str());

	/* these last two are in the form #d#+#  .. i need the three #'s */
	temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
	
	hit_dice = atoi(temp2.substr(0,temp2.find("d")).c_str());
	hit_size = atoi(temp2.substr(temp2.find("d")+1,temp2.find("+")).c_str());
	hit_bonus = atoi(temp2.substr(temp2.find("+")+1).c_str());

	temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
	
	attack_dice = atoi(temp2.substr(0,temp2.find("d")).c_str());
	attack_size = atoi(temp2.substr(temp2.find("d")+1,temp2.find("+")).c_str());
	attack_bonus = atoi(temp2.substr(temp2.find("+")+1).c_str());

	cf->get_line(&temp);

	temp2=temp.substr(0,temp.find(" "));
	gold=atoi(temp2.c_str());

	temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
  experience=atoi(temp2.c_str());

  cf->get_line(&temp);

	temp2=temp.substr(0,temp.find(" "));
	load_pos=atoi(temp2.c_str());

	temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
  default_pos=atoi(temp2.c_str());

	temp = temp.substr(temp.find(" ")+1);
	temp2=temp.substr(0,temp.find(" "));
  gender=atoi(temp2.c_str());

	if(!type ) {
		return TRUE;
	}

	/* espec mob */
	cf->get_line(&temp);
	while(!cf->eof() && !temp.empty() && temp.at(0) != 'E' && temp.at(0) != '$') {
		temp2=temp.substr(0,temp.find(":"));
		if(!(_stricmp("BareHandAttack",temp2.c_str()))) {
			 attack_type=atoi(temp.substr(temp.find(":")+2).c_str());
		} else if(!(_stricmp("Str",temp2.c_str()))) {
			stats[STAT_STR]=atoi(temp.substr(temp.find(":")+2).c_str());
		} else if(!(_stricmp("StrAdd",temp2.c_str()))) {
			stats[STAT_STRADD]=atoi(temp.substr(temp.find(":")+2).c_str());
		} else if(!(_stricmp("Con",temp2.c_str()))) {
			stats[STAT_CON]=atoi(temp.substr(temp.find(":")+2).c_str());
		} else if(!(_stricmp("Dex",temp2.c_str()))) {
			stats[STAT_DEX]=atoi(temp.substr(temp.find(":")+2).c_str());
		} else if(!(_stricmp("Wis",temp2.c_str()))) {
			stats[STAT_WIS]=atoi(temp.substr(temp.find(":")+2).c_str());
		} else if(!(_stricmp("Int",temp2.c_str()))) {
			stats[STAT_INT]=atoi(temp.substr(temp.find(":")+2).c_str());
		} else if(!(_stricmp("Cha",temp2.c_str()))) {
			stats[STAT_CHA]=atoi(temp.substr(temp.find(":")+2).c_str());
		}
		cf->get_line(&temp);
	}

  return TRUE;
}

extern const char *stat_names[];

void frigfrag(string *sex);

bool Mob::write_mob(CircleFile *cf)
{
  string obuf;
  string temp;
  char buf[33];
  int i;

  obuf = iprintf("#%d\n",number);

	obuf += iprintf("%s~\n",alias.c_str());
  obuf += iprintf("%s~\n",sdesc.c_str());
  obuf += iprintf("%s\n~\n",ldesc.c_str());
	temp=ddesc;
	frigfrag(&temp);
	obuf += iprintf("%s\n~\n",temp.c_str());

  bitvector2ascii(buf, action);
  obuf += iprintf("%s ",buf);
	bitvector2ascii(buf, affect);
	obuf+= iprintf("%s %d E\n",buf,alignment);
	obuf += iprintf ("%d %d %d %dd%d+%d %dd%d+%d\n",
		level, thac0, ac, hit_dice,hit_size,hit_bonus,attack_dice,attack_size,attack_bonus);
	obuf+= iprintf("%d %d\n%d %d %d\n",gold, experience, load_pos,default_pos,gender);

	if(attack_type) {
		obuf += iprintf("BareHandAttack: %d\n",attack_type);
	}

	for(i=0;i<NUM_OF_STATS;i++) {
		if(stats[i] != 14) {
			if(i == STAT_STRADD && stats[i] != 0) {
  			obuf += iprintf("%s: %d\n",stat_names[i],stats[i]);
			}
		}
	}
  obuf += iprintf("E\n");
  cf->write_line(obuf);

	return FALSE;
}
